//------------------------------------------------------------
#include "RODOS5_6.h"
#include <iostream>
#include <string>
#include <sstream>
#include <iomanip>
//------------------------------------------------------------
const uint8_t RODOS5_6_MAXIM_CRC8_TABLE[256] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53 };
//------------------------------------------------------------
bool RODOS56_t::CRC8_RomCheck(uint64_t Rom)
{
 uint8_t crc = 0;
 for (uint8_t I=0; I<7; I++) { crc = RODOS5_6_MAXIM_CRC8_TABLE[ crc ^ (uint8_t)(Rom>>(I*8)) ]; }
 if ( (uint8_t)(Rom >> (7*8)) == crc ) { return true;  }
 return false;
}
//------------------------------------------------------------
uint8_t RODOS56_t::CRC8(uint8_t CRC_IN, uint8_t Data)
{
 return RODOS5_6_MAXIM_CRC8_TABLE[CRC_IN ^ Data];
}
//------------------------------------------------------------
bool RODOS56_t::READ_ROM(uint64_t *ROM64)
{   //  чтение ROM
 uint32_t B_H;
 uint32_t B_L;
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( OW_RESET() && OW_WRITE_BYTE(0x33) && OW_READ_4BYTE(&B_L) && OW_READ_4BYTE(&B_H) )
		{
		 uint64_t ROM = ((uint64_t)B_H << 32) | B_L;
		 if ( this->CRC8_RomCheck(ROM) )
			{
			 *ROM64 = ROM;
			 return true;
			}
		}
	}
 return false;
}
//------------------------------------------------------------
bool RODOS56_t::MATCH_ROM(uint64_t ROM)
{   //  выбор прибора по ROM
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( OW_RESET() && OW_WRITE_BYTE(0x55) && OW_WRITE_4BYTE((uint32_t)ROM) && OW_WRITE_4BYTE((uint32_t)(ROM>>32)) )
		{
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool RODOS56_t::SKIP_ROM()
{
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( OW_RESET() && OW_WRITE_BYTE(0xCC) ) { return true; }
	}
 return false;
}
//------------------------------------------------------------
bool RODOS56_t::SEARCH_ROM()
{   //  поиск ROM
 this->ONEWIRE_COUNT     = 0;
 uint8_t LastDiscrepancy = 0;
 uint64_t SnRom = 0;
 uint8_t DiscrepancyMarker;
 uint8_t BitIndex;
 uint8_t TwoBits;
 uint8_t RomBit;
 do {
	 if ( OW_RESET() && OW_WRITE_BYTE(0xF0) )
		{
		 DiscrepancyMarker = 0;
		 for (BitIndex = 1; BitIndex < 65; BitIndex++)
			{
			 if ( OW_READ_2BIT(&TwoBits) == false ) { return false; }
			 switch (TwoBits)
				{
				 case (0x00) : {
						if      ( BitIndex == LastDiscrepancy ) { RomBit = 1; }
						else if ( BitIndex > LastDiscrepancy )  { RomBit = 0; DiscrepancyMarker = BitIndex; }
						else {
							  RomBit = (SnRom >> (BitIndex-1)) & 0x01;
							  if ( RomBit == 0 ) { DiscrepancyMarker = BitIndex; }
							 }
						break; }
				 case (0x01) : { RomBit = 1; break; }
				 case (0x02) : { RomBit = 0; break; }
				 default :     { return false; }
				}
			 if ( RomBit != 0 ) { SnRom |=   (1ULL << (BitIndex-1)); }
			 else			    { SnRom &=  ~(1ULL << (BitIndex-1)); }
			 if ( OW_WRITE_BIT(RomBit) == false ) { return false; }
			}
		 LastDiscrepancy = DiscrepancyMarker;
		 if ( this->CRC8_RomCheck(SnRom) ) { this->ONEWIRE_ROM[this->ONEWIRE_COUNT++] = SnRom; }
		 else                              { return false; }
		}
		else { return false; }
	}
	while ( ( LastDiscrepancy != 0) && (this->ONEWIRE_COUNT < RODOS5_6_MAX_SENS_COUNT) );
 return true;
}
//------------------------------------------------------------
bool RODOS56_t::SKIP_ROM_CONVERT()
{   //  пропуск ROM-команд, старт измерения температуры, 9ms
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( OW_RESET() && OW_WRITE_BYTE(0xCC) && OW_WRITE_BYTE(0x44) ) { return true; }
	}
 return false;
}
//------------------------------------------------------------
bool RODOS56_t::GET_TEMPERATURE(uint64_t ROM, float *T)
{   //  чтение температуры
 uint32_t L1, L2;
 uint8_t L3;
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( MATCH_ROM(ROM) && OW_WRITE_BYTE(0xBE) && OW_READ_4BYTE(&L1) && OW_READ_4BYTE(&L2) && OW_READ_BYTE(&L3) )
		{
		 uint8_t CRC = 0;
		 for ( uint8_t I=0; I<4; I++) { CRC = this->CRC8( CRC, (uint8_t)(L1 >> (I*8)) ); }
		 for ( uint8_t I=0; I<4; I++) { CRC = this->CRC8( CRC, (uint8_t)(L2 >> (I*8)) ); }
		 if ( CRC == L3 )
			{
			 switch ( ROM & 0xFF )		// Family Code
				{	
				 case (0x28) : { *T = DS18B20_CalcTemp((uint8_t)(L1 >> 8), (uint8_t)L1, (uint8_t)L2); return true; }
				 case (0x10) : { *T = DS18S20_CalcTemp((uint8_t)(L1 >> 8), (uint8_t)L1);              return true; }
				 default :     {}
				}
			}
		}
	}
 return false;
}
//------------------------------------------------------------
void RODOS56_t::READ_TEMPERATURE(void)
{
 if ( this->ONEWIRE_COUNT == 0 )
	{
	 uint8_t Status = 0;
	 for (uint8_t TryCount = 3; TryCount >0; TryCount--)
		{
		 if ( (this->SEARCH_ROM() == true) && (this->ONEWIRE_COUNT != 0) ) { Status = 1; break; }
		}
	 if ( Status == 0 )
		{
		 std::cout << "ERROR: Find Sensors Error" << std::endl;
		 return;
		}
	}
 
 if ( SKIP_ROM_CONVERT() == true )
	{	 
	 usb_t::Sleep(1000);
	 for (uint32_t I=0; I < this->ONEWIRE_COUNT; I++)
		{
		 if (this->GET_TEMPERATURE(this->ONEWIRE_ROM[I], &this->ONEWIRE_TEMP[I]))
			{
			 std::cout << "ROM=" << std::setfill('0') << std::setw(16) << std::hex << this->ONEWIRE_ROM[I] << " T=" << std::fixed << std::setprecision(4) << this->ONEWIRE_TEMP[I] << std::endl;
			}
			else { std::cout << "ROM=" << std::setfill('0') << std::setw(16) << std::hex << this->ONEWIRE_ROM[I] << "ERROR: Get Temperature Error" << std::endl; }
		}
	}
	else { std::cout << "ERROR: Skip ROM Command Error" << std::endl; }
}
//------------------------------------------------------------
bool RODOS56_t::PORTS(enum RODOS5_6_PORTS_COMMANDS COMMAND)
{   //  PORTB3 - 0x08
    //  PORTB4 - 0x10
 uint8_t PS;
 if ( USB_GET_PORT(&PS) )
	{
	 switch ( COMMAND )
		{
		 case (PORT1ON)  : { return USB_SET_PORT(PS|0x08); }	 //  включение / выключение
		 case (PORT1OFF) : { return USB_SET_PORT(PS&0x10); }
		 case (PORT2ON)  : { return USB_SET_PORT(PS|0x10); }
		 case (PORT2OFF) : { return USB_SET_PORT(PS&0x08); }
		 case (PORTSSET) : { return USB_EE_WR(0x04, PS);   }	 //  запись состояния при включении
		 default :         {}
		}
	}
 return false;
}
//------------------------------------------------------------
void RODOS56_t::TERMOSTATE_READ()
{   //  чтение границ температуры из USB
 uint8_t H,L;
 uint16_t S;
 if ( USB_EE_RD(0x10, &H) && USB_EE_RD(0x11, &L) ) { std::cout << "T1H: " + std::to_string((float)(((uint16_t)H<<8)|L)/16.0) << std::endl; }
 else                                              { std::cout << "T1H: ERR" << std::endl; }
 if ( USB_EE_RD(0x12, &H) && USB_EE_RD(0x13, &L) ) { std::cout << "T1L: " + std::to_string((float)(((uint16_t)H<<8)|L)/16.0) << std::endl; }
 else                                              { std::cout << "T1L: ERR" << std::endl; }
 if ( USB_EE_RD(0x14, &H) && USB_EE_RD(0x15, &L) ) { std::cout << "T2H: " + std::to_string((float)(((uint16_t)H<<8)|L)/16.0) << std::endl; }
 else                                              { std::cout << "T2H: ERR" << std::endl; }
 if ( USB_EE_RD(0x16, &H) && USB_EE_RD(0x17, &L) ) { std::cout << "T2L: " + std::to_string((float)(((uint16_t)H<<8)|L)/16.0) << std::endl; }
 else                                              { std::cout << "T2L: ERR" << std::endl; }
}
//------------------------------------------------------------
bool RODOS56_t::TERMOSTATE_WRITE(enum RODOS5_6_THREMOSTAT_THR TS, float Ftemp)
{ //  запись границ температуры в USB
 uint16_t Stemp = (uint16_t)(Ftemp * 16.0);
 switch (TS)
	{
     case T1H: { return ( USB_EE_WR(0x10, (uint8_t)(Stemp>>8)) && USB_EE_WR(0x11, (uint8_t)Stemp) ); }
     case T1L: { return ( USB_EE_WR(0x12, (uint8_t)(Stemp>>8)) && USB_EE_WR(0x13, (uint8_t)Stemp) ); }
     case T2H: { return ( USB_EE_WR(0x14, (uint8_t)(Stemp>>8)) && USB_EE_WR(0x15, (uint8_t)Stemp) ); }
     case T2L: { return ( USB_EE_WR(0x16, (uint8_t)(Stemp>>8)) && USB_EE_WR(0x17, (uint8_t)Stemp) ); }
     default:  {}
    }
 return false;
}
//------------------------------------------------------------
bool RODOS56_t::TERMOSTATE_SET(uint64_t ROM)
{
 for (uint8_t Byte=0; Byte < 8; Byte++)
	{
	 if ( USB_EE_WR( 0x08+Byte, (uint8_t)(ROM >> (Byte*8)) ) == true ) { return true; }
	}
 return false;
}
//------------------------------------------------------------
bool RODOS56_t::AUTONOM_DELAY_SET(uint8_t Delay)
{
 return USB_EE_WR(0x05, Delay);
}
//------------------------------------------------------------
bool RODOS56_t::AUTONOM_DELAY_READ(uint8_t *Delay)
{
 return USB_EE_RD(0x05, Delay);
}
//------------------------------------------------------------
float RODOS56_t::DS18B20_CalcTemp(uint8_t TempMSB, uint8_t TempLSB, uint8_t Config)
{
 // Two's complement
 uint8_t  Sign     =  TempMSB & 0x80;
 uint16_t TempCode = (uint16_t)( ((uint16_t)TempMSB << 8) | TempLSB );
 if ( Sign != 0 ) { TempCode = (uint16_t)((uint16_t)(~TempCode) + 1U); }

 switch (Config & DS18B20_SKRATCHPAD_CONF_MASK)
	 {
	  case (DS18B20_SKRATCHPAD_CONF_RES_9BIT)  : { TempCode &= 0xFFF8; break; }
	  case (DS18B20_SKRATCHPAD_CONF_RES_10BIT) : { TempCode &= 0xFFFC; break; }
	  case (DS18B20_SKRATCHPAD_CONF_RES_11BIT) : { TempCode &= 0xFFFE; break; }
	  case (DS18B20_SKRATCHPAD_CONF_RES_12BIT) :
	  default : {}
	 }

 if ( Sign == 0 ) { return (float)TempCode * DS18B20_SKRATCHPAD_12BIT_MULT_FL; }
 else             { return (float)(-(float)TempCode) * DS18B20_SKRATCHPAD_12BIT_MULT_FL; }
}
//------------------------------------------------------------
float RODOS56_t::DS18S20_CalcTemp(uint8_t TempMSB, uint8_t TempLSB)
{
 int16_t TempCode = (int16_t)( ((uint16_t)TempMSB << 8) | TempLSB );
 return (float)TempCode * 0.5;
}
//------------------------------------------------------------
uint32_t RODOS56_t::GetSensCount(void)
{
 return this->ONEWIRE_COUNT;
}
//------------------------------------------------------------
uint64_t RODOS56_t::GetSensRom(uint32_t Pos)
{
 if ( Pos < this->ONEWIRE_COUNT ) { return this->ONEWIRE_ROM[Pos]; }
 return 0;
}
//------------------------------------------------------------
