//------------------------------------------------------------
#ifndef RODOS5_6_H
#define RODOS5_6_H
//------------------------------------------------------------
#include "onewire.h"
//------------------------------------------------------------
enum RODOS5_6_PORTS_COMMANDS {
	PORT1ON,
	PORT1OFF,
	PORT2ON,
	PORT2OFF,
	PORTSSET
};
enum RODOS5_6_THREMOSTAT_THR {
	T1H, 
	T1L,
	T2H,
	T2L
};
//------------------------------------------------------------
#define DS18B20_SKRATCHPAD_CONF_MASK			0x60U
#define DS18B20_SKRATCHPAD_CONF_RES_9BIT		0x00U
#define DS18B20_SKRATCHPAD_CONF_RES_10BIT		0x20U
#define DS18B20_SKRATCHPAD_CONF_RES_11BIT		0x40U
#define DS18B20_SKRATCHPAD_CONF_RES_12BIT		0x60U

#define DS18B20_SKRATCHPAD_12BIT_MULT_E4		625L
#define DS18B20_SKRATCHPAD_12BIT_MULT_FL		0.0625
//------------------------------------------------------------
#define RODOS5_6_MAX_SENS_COUNT					128
//------------------------------------------------------------
class RODOS56_t : public OneWire_t
{
public:
    RODOS56_t (uint16_t VID, uint16_t PID) : OneWire_t(VID, PID) { this->ONEWIRE_COUNT = 0; }
    RODOS56_t (char *path) : OneWire_t(path) { this->ONEWIRE_COUNT = 0; }
    ~RODOS56_t () {}

private:
	bool    CRC8_RomCheck(uint64_t Rom);
	uint8_t CRC8(uint8_t CRC_IN, uint8_t Data);

    bool READ_ROM(uint64_t *ROM64);
    bool MATCH_ROM(uint64_t ROM);
    bool SKIP_ROM();

    bool SKIP_ROM_CONVERT();
    bool GET_TEMPERATURE(uint64_t ROM, float *T);
	
	float DS18B20_CalcTemp(uint8_t TempMSB, uint8_t TempLSB, uint8_t Config);
	float DS18S20_CalcTemp(uint8_t TempMSB, uint8_t TempLSB);

public:
    bool SEARCH_ROM(void);
    void READ_TEMPERATURE(void);
    bool PORTS(enum RODOS5_6_PORTS_COMMANDS COMMAND);

    void TERMOSTATE_READ(void);
    bool TERMOSTATE_WRITE(enum RODOS5_6_THREMOSTAT_THR TS, float Ftemp);
    bool TERMOSTATE_SET(uint64_t ROM);
	
	bool AUTONOM_DELAY_SET(uint8_t Delay);
	bool AUTONOM_DELAY_READ(uint8_t *Delay);
	
	uint32_t GetSensCount(void);
	uint64_t GetSensRom(uint32_t Pos);

public:
    uint32_t   ONEWIRE_COUNT;            					//  количество датчиков
private:
    uint64_t  ONEWIRE_ROM[RODOS5_6_MAX_SENS_COUNT];         //  номера ROM
    float     ONEWIRE_TEMP[RODOS5_6_MAX_SENS_COUNT];        //  температура

};
//------------------------------------------------------------
#endif // RODOS5_6_H
//------------------------------------------------------------
