//  Copyright 2019 SILINES. Released under GPLv3 in Russia.
//---------------------------------------------------------------------------------------------------
//  Пример программы работы с устройством RODOS-5 и RODOS-6. Версия 1.04
//---------------------------------------------------------------------------------------------------
//  www.silines.ru
//---------------------------------------------------------------------------------------------------
#define R56_VID 0x20A0
#define R56_PID 0x4173
//---------------------------------------------------------------------------------------------------
#include <iostream>
#include <stdio.h>
#include <cstring>
#include <unistd.h>
#include <getopt.h>
#include <string>
#include <cstdlib>
#include "RODOS5_6.h"
#include <memory>
#include <iomanip>
#include <sstream>
using namespace std;
//---------------------------------------------------------------------------------------------------
struct flags_t
{
    bool no_flags;
    bool all;
    bool read;
    bool ts_read;
    bool ts_set;

    unsigned int device_id;

    bool p1_on;
    bool p1_off;
    bool p2_on;
    bool p2_off;

    bool portset;

    float T1H;
    float T1L;
    float T2H;
    float T2L;

    uint64_t default_ROM;
	
	bool    AutDelayWR;
	uint8_t AutDelayWR_Value;
	bool    AutDelayRD;

    void init (void);
};
//---------------------------------------------------------------------------------------------------
int main(int argc, char* argv[])
{
    std::ios::sync_with_stdio(false);

    int MAIN_RESULT = 1;

    flags_t flags;
    flags.init();

    //  Разбор переданных параметров
    const char* short_options = "hratsji:n:f:d:m:w:x:y:z:";

        const struct option long_options[] = {
            {"help",no_argument,NULL,'h'},
            {"id",required_argument,NULL,'i'},
            {"on",required_argument,NULL,'n'},
            {"off",required_argument,NULL,'f'},
            {"read",no_argument,NULL,'r'},
            {"all",no_argument,NULL,'a'},
            {"set",no_argument,NULL,'s'},
            {"rts",no_argument,NULL,'t'},
            {"t1h",required_argument,NULL,'w'},
            {"t1l",required_argument,NULL,'x'},
            {"t2h",required_argument,NULL,'y'},
            {"t2l",required_argument,NULL,'z'},
            {"default",required_argument,NULL,'d'},
			{"atwr",required_argument,NULL,'m'},
			{"atrd",no_argument,NULL,'j'},
            {NULL,0,NULL,0}
        };

        int rez;
        int option_index;

        while ((rez=getopt_long(argc,argv,short_options,long_options,&option_index))!=-1)
        {
            switch(rez){
                case 'h': {
                        cout << " Используйте параметры\n"
                                "   --help, -h - вызов справки\n"
                                "\n"
                                "   --id ID   - обращение к RODOS-5/RODOS-6 с соответствующим ID\n"
                                "   --all, -a - для выполнения на всех RODOS-5/RODOS-6\n"
                                "\n"
                                "   --read, -r - считать температуру с датчиков\n"
                                "\n"
                                "   --on p1/p2, -n p1/p2  - изменить состояние выхода p1 или p2 на вкл\n"
                                "   --off p1/p2, -f p1/p2 - изменить состояние выхода p1 или p2 на откл\n"
                                "   --set, -s - записать состояния выходов при подаче питания\n"
                                "\n"
                                "   --rts, -t  - прочитать значения термостата\n"
                                "   --t1h temp - установить температуру термостата\n"
                                "   --t1l temp - установить температуру термостата\n"
                                "   --t2h temp - установить температуру термостата\n"
                                "   --t2l temp - установить температуру термостата\n"
                                "   --default ROM, -d ROM - установить датчик с соответствующим ROM\n"
                                "                           для автономного термостатирования\n"
                                "   --atwr [delay], -m [delay] - установить задержку перехода в автономный режим\n"
                                "                                допустимые значения 5...255; 0 - отключить\n"
                                "   --atrd, -j[delay] - прочитать значение задержки перехода в автономный режим\n"
                                "                       0 - отключена\n"
                                "   или запустите без параметров для получения списка всех устройств и их ID\n"
                         << flush;
                    flags.no_flags = false;
                    break;
                }
                case 'i': {
					try { flags.device_id = stol(optarg); }
                    catch (...) { cout <<  "Ошибка: некорректное значение ID" << endl; return -1; }
                    flags.no_flags = false;
                    break;
                }
                case 'n': {
                    if (!strcmp(optarg, "p1")) flags.p1_on = true;
                    if (!strcmp(optarg, "p2")) flags.p2_on = true;
                    flags.no_flags = false;
                    break;
                }
                case 'f': {
                    if (!strcmp(optarg, "p1")) flags.p1_off = true;
                    if (!strcmp(optarg, "p2")) flags.p2_off = true;
                    flags.no_flags = false;
                    break;
                }
                case 'r': {
                    flags.read = true;
                    flags.no_flags = false;
                    break;
                }
                case 'a': {
                    flags.all = true;
                    flags.no_flags = false;
                    break;
                }
                case 's': {
                    flags.portset = true;
                    flags.no_flags = false;
                    break;
                }
                case 't': {
                    flags.ts_read = true;
                    flags.no_flags = false;
                    break;
                }

                case 'w': {
                    flags.ts_set = true;
                    flags.T1H = static_cast<float>(atof(optarg));
                    flags.no_flags = false;
                    break;
                }

                case 'x': {
                    flags.ts_set = true;
                    flags.T1L = static_cast<float>(atof(optarg));
                    flags.no_flags = false;
                    break;
                }

                case 'y': {
                    flags.ts_set = true;
                    flags.T2H = static_cast<float>(atof(optarg));
                    flags.no_flags = false;
                    break;
                }

                case 'z': {
                    flags.ts_set = true;
                    flags.T2L = static_cast<float>(atof(optarg));
                    flags.no_flags = false;
                    break;
                }

                case 'd': {
                    flags.default_ROM = strtoull(optarg, nullptr, 16);
                    if (flags.default_ROM == 0) cout << "ROM указан неверно" << endl;
                    flags.no_flags = false;
                    break;
                }
                
				case 'm': {
					try {
						 int AutonomDelayTemp = std::stoi(optarg);
						 if ( (AutonomDelayTemp == 0 ) || ((AutonomDelayTemp >= 5) && (AutonomDelayTemp <= 255)) )
							{
							 flags.AutDelayWR_Value = (uint8_t)AutonomDelayTemp;
							 flags.AutDelayWR = true;
							 flags.no_flags = false;
							}
							else { cout << "Введено некорректное значение задержки" << endl; return -1; }
						}
						catch (...) { cout << "Введено некорректное значение задержки" << endl; return -1; }
						
                    break;
                }
                
				case 'j': {
                    flags.AutDelayRD = true;
                    flags.no_flags = false;
                    break;
                }
				
				case '?': default: {
                    cout << "Неизвестный параметр" << endl;
                    break;
                }
            }
        }

    if (((flags.p1_on)&&(flags.p1_off)) || ((flags.p2_on)&&(flags.p2_off)) || ((flags.all)&&(flags.device_id))) { cout << "Неверно заданы параметры\n" << endl; MAIN_RESULT = -1; }
    else
    {
        //  Подключение к RODOS-5/RODOS-6 и исполнение команд
        struct hid_device_info *devs;
        int device_counter = 0;

        devs = hid_enumerate(R56_VID, R56_PID);

        if (flags.no_flags) cout << "Поиск устройств..." << endl;
		
		RODOS56_t *R56 = NULL;
        while (devs)
        {
			try { R56 = new RODOS56_t(devs->path); }
			catch(...) { cout << "Memory allocation failure!" << endl; return -1;}
			
            if (R56->CHECK_HANDLE() == false) cout << "Ошибка подключения" << endl;
            else
            {
                wstring PrString;
                R56->Read_Device_Production_String(PrString);

                if (flags.no_flags) {
                    wcout << PrString << flush;
                    cout << " ID: " << R56->Get_Device_ID() << endl;
                }

                if ((!wcscmp(PrString.c_str(), L"RODOS-5"))||(!wcscmp(PrString.c_str(), L"RODOS-6")))
                {
                    device_counter++;

                    if (flags.no_flags)
                    {
                        R56->SEARCH_ROM();
                        if ( R56->ONEWIRE_COUNT == 0 ) { cout << " Датчики не найдены" << endl; }
                        else
							{
							 cout << " Найдено DALLAS - " << R56->ONEWIRE_COUNT << endl;
							 for ( uint32_t I=0; I<R56->GetSensCount(); I++)
								{
								 std::cout << "S"  + std::to_string(I+1) + ": " << std::setfill('0') << std::setw(16) << std::hex << R56->GetSensRom(I) << std::endl;
								}
							}
                    }

                    if (((flags.device_id == 0)&&(flags.all)) || (flags.device_id == R56->Get_Device_ID()))
                    {
                        cout << "ID: " << R56->Get_Device_ID() << " - " << endl;

                        if (flags.p1_on) 
							{
							 if ( R56->PORTS(PORT1ON) ) { cout << "PORT1 ON: OK" << endl; }
							 else                       { cout << "PORT1 ON: ERR" << endl; }
							}
                        if (flags.p1_off)
							{
							 if ( R56->PORTS(PORT1OFF) ) { cout << "PORT1 OFF: OK" << endl; }
							 else                        { cout << "PORT1 OFF: ERR" << endl; }
							}
                        if (flags.p2_on)
							{
							 if ( R56->PORTS(PORT2ON) ) { cout << "PORT2 ON: OK" << endl; }
							 else                       { cout << "PORT2 ON: ERR" << endl; }
							}
                        if (flags.p2_off)
							{
							 if ( R56->PORTS(PORT2OFF) ) { cout << "PORT2 OFF: OK" << endl; }
							 else                        { cout << "PORT2 OFF: ERR" << endl; }
							}
                        if (flags.portset)
							{
							 if ( R56->PORTS(PORTSSET) ) { cout << "PORT SET: OK" << endl; }
							 else                        { cout << "PORT SET: ERR" << endl; }
							}
                        if (flags.read)  { R56->READ_TEMPERATURE(); }
                        if (flags.ts_set)
							{
                            bool result = true;
                            if ( (flags.T1H != -1000) && R56->TERMOSTATE_WRITE(T1H, flags.T1H) ) { result = false; }
                            if ( (flags.T1L != -1000) && R56->TERMOSTATE_WRITE(T1L, flags.T1L) ) { result = false; }
                            if ( (flags.T2H != -1000) && R56->TERMOSTATE_WRITE(T2H, flags.T2H) ) { result = false; }
                            if ( (flags.T2L != -1000) && R56->TERMOSTATE_WRITE(T2L, flags.T2L) ) { result = false; }
                            if (result) { cout << "Границы температуры записаны успешно" << endl; }
                            else        { cout << "Ошибка записи границ температуры!"    << endl; }
							}
                        if (flags.ts_read) { R56->TERMOSTATE_READ(); }
                        if (flags.default_ROM)
							{
                            if (R56->TERMOSTATE_SET(flags.default_ROM)) cout << "Номер датчика записан" << endl;
                            else cout << "Ошибка записи номера датчика!" << endl;
							}
						if ( flags.AutDelayWR )
							{
							 if ( R56->AUTONOM_DELAY_SET(flags.AutDelayWR_Value) ) { cout << "Новое значение задержки перехода в автономный режим записано" << endl; }
							 else { cout << "Ошибка записи нового значения задержки перехода в автономный режим" << endl; }
							}
						if ( flags.AutDelayRD )
							{
							 uint8_t DelayValue;
							 if ( R56->AUTONOM_DELAY_READ(&DelayValue) == true ) { cout << "Delay = " << std::to_string(DelayValue) << endl; }
							 else                    { cout << "Не удалось прочитать значение задержки перехода в автономный режим" << endl; }
							}
                    }
                }
            }
            devs = devs->next;
			delete R56;
			R56 = NULL;
        }
        if (flags.no_flags) cout << "Найдено RODOS-5/RODOS-6: " << device_counter << endl;
    }

    return MAIN_RESULT;
}

void flags_t::init(void)
{
    no_flags = true;

    all = false;
    read = false;
    ts_read = false;
    ts_set = false;

    p1_on = false;
    p1_off = false;
    p2_on = false;
    p2_off = false;
    portset = false;

    T1H = -1000;
    T1L = -1000;
    T2H = -1000;
    T2L = -1000;

    device_id = 0;

    default_ROM = 0ULL;
	
	AutDelayWR = false;
	AutDelayWR_Value = 0x00;
	AutDelayRD = false;
}
