//------------------------------------------------------------
#include "onewire.h"
#include <iostream>
#include <unistd.h>
//------------------------------------------------------------
bool OneWire_t::OW_RESET()
{
 USB_BUF_CLEAR();
 USB_BUFO[1]=0x18;
 USB_BUFO[2]=0x48;
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( USB_SET_FEATURE() )
		{
		 usb_t::Sleep(10);
		 if ( USB_GET_FEATURE() &&  (USB_BUFI[1]==0x18) && (USB_BUFI[2]==0x48) && (USB_BUFI[3]==0x00) )
			{
			 return true;
			}
		}
	}
 return false;
}
//------------------------------------------------------------
bool OneWire_t::OW_READ_BIT(uint8_t *B)
{
 USB_BUF_CLEAR();
 USB_BUFO[1]=0x18;
 USB_BUFO[2]=0x81;
 USB_BUFO[3]=0x01;
 if ( USB_SET_FEATURE() )
	{
	 usb_t::Sleep(5);
	 if ( USB_GET_FEATURE() && (USB_BUFI[1]==0x18) && (USB_BUFI[2]==0x81) )
		{
		 *B = USB_BUFI[3] & 0x01;
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool OneWire_t::OW_READ_2BIT(uint8_t *B)
{
 USB_BUF_CLEAR();
 USB_BUFO[1]=0x18;
 USB_BUFO[2]=0x82;
 USB_BUFO[3]=0x01;
 USB_BUFO[4]=0x01;
 if (USB_SET_FEATURE())
	{
	 usb_t::Sleep(5);
     if ( USB_GET_FEATURE() && (USB_BUFI[1]==0x18) && (USB_BUFI[2]==0x82) )
		{
		 *B = (USB_BUFI[3] & 0x01) | ((USB_BUFI[4] & 0x01) << 1);
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool OneWire_t::OW_READ_BYTE(uint8_t *B)
{
 USB_BUF_CLEAR();
 USB_BUFO[1]=0x18;
 USB_BUFO[2]=0x88;
 USB_BUFO[3]=0xFF;
 if (USB_SET_FEATURE())
	{
     usb_t::Sleep(5);
     if ( USB_GET_FEATURE() && (USB_BUFI[1]==0x18) && (USB_BUFI[2]==0x88) )
		{
		 *B = USB_BUFI[3];
		 return true;
		}
    }
 return false;
}
//------------------------------------------------------------
bool OneWire_t::OW_READ_4BYTE(uint32_t *B)
{
 USB_BUF_CLEAR();
 USB_BUFO[1]=0x18;
 USB_BUFO[2]=0x84;
 USB_BUFO[3]=0xFF;
 USB_BUFO[4]=0xFF;
 USB_BUFO[5]=0xFF;
 USB_BUFO[6]=0xFF;
 if (USB_SET_FEATURE())
	{
     usb_t::Sleep(30);
     if ( USB_GET_FEATURE() && (USB_BUFI[1]==0x18) && (USB_BUFI[2]==0x84) )
		{
		 *B = (uint32_t)USB_BUFI[3] | ((uint32_t)USB_BUFI[4]<<8) | ((uint32_t)USB_BUFI[5]<<16) | ((uint32_t)USB_BUFI[6]<<24);
		 return true;
		}
    }
 return false;
}
//------------------------------------------------------------
bool OneWire_t::OW_WRITE_BIT(uint8_t B)
{
 USB_BUF_CLEAR();
 USB_BUFO[1] = 0x18;
 USB_BUFO[2] = 0x81;
 USB_BUFO[3] = B & 0x01;
 if (USB_SET_FEATURE())
	{
     usb_t::Sleep(5);
     if ( USB_GET_FEATURE() && (USB_BUFI[1]==0x18) && (USB_BUFI[2]==0x81) && ((USB_BUFI[3]&0x01)==(B&0x01)) )
		{
		 return true;
		}
    }
 return false;
}
//------------------------------------------------------------
bool OneWire_t::OW_WRITE_BYTE(uint8_t B)
{
 USB_BUF_CLEAR();
 USB_BUFO[1] = 0x18;
 USB_BUFO[2] = 0x88;
 USB_BUFO[3] = B;
 if (USB_SET_FEATURE())
	{
     usb_t::Sleep(5);
     if ( USB_GET_FEATURE() && (USB_BUFI[1]==0x18) && (USB_BUFI[2]==0x88) && (USB_BUFI[3]==B) )
		{
		 return true;
		}
    }
 return false;
}
//------------------------------------------------------------
bool OneWire_t::OW_WRITE_4BYTE(uint32_t B)
{
 uint8_t D0 = (uint8_t)B;
 uint8_t D1 = (uint8_t)(B>>8);
 uint8_t D2 = (uint8_t)(B>>16);
 uint8_t D3 = (uint8_t)(B>>24);
 USB_BUF_CLEAR();
 USB_BUFO[1] =0x18;
 USB_BUFO[2] =0x84;
 USB_BUFO[3] = D0;
 USB_BUFO[4] = D1;
 USB_BUFO[5] = D2;
 USB_BUFO[6] = D3;
 if (USB_SET_FEATURE())
	{
	 usb_t::Sleep(30);
     if ( 	USB_GET_FEATURE()   && 
			(USB_BUFI[1]==0x18) &&
			(USB_BUFI[2]==0x84) &&
			(USB_BUFI[3]==D0)   &&
			(USB_BUFI[4]==D1)   &&
			(USB_BUFI[5]==D2)   &&
			(USB_BUFI[6]==D3)	) { return true; }
	}
 return false;
}
//------------------------------------------------------------
