#include "usb.h"
#include <iostream>
#include <unistd.h>
//------------------------------------------------------------
usb_t::usb_t(uint16_t VID, uint16_t PID)
{
 handle = hid_open(VID, PID, NULL);
 if (handle)
	{
     hid_get_product_string (handle, Product_String, USB_T_PRODUCT_STR_MAX_SIZE);
     USB_GET_ID(&Device_ID);
    }
}
//------------------------------------------------------------
usb_t::usb_t(char *hid_device_path)
{
 handle = hid_open_path(hid_device_path);
 if (handle)
	{
     hid_get_product_string (handle, Product_String, USB_T_PRODUCT_STR_MAX_SIZE);
     USB_GET_ID(&Device_ID);
    }
}
//------------------------------------------------------------
usb_t::~usb_t()
{
 if (handle != NULL) { hid_close(handle); }
}
//------------------------------------------------------------
void usb_t::Sleep(int ms)
{
 usleep(ms*1000);
}
//------------------------------------------------------------
bool usb_t::CHECK_HANDLE()
{
 if (handle == NULL) { return false; }
 return true;
}
//------------------------------------------------------------
void usb_t::USB_BUF_CLEAR()
{
 for (uint8_t i=0; i < USB_T_RX_TX_BUF_SIZE; i++) { USB_BUFI[i]=0; USB_BUFO[i]=0; } //  очистка буферов приёма и передачи
}
//------------------------------------------------------------
bool usb_t::USB_GET_FEATURE()
{   //  чтение в буфер из устройства
 for (uint8_t TryCnt = 3; TryCnt > 0; TryCnt--)
	{
	 try {
		  if ( hid_get_feature_report(handle, USB_BUFI, USB_T_RX_TX_BUF_SIZE) != (-1) ) { return true; }
		 }
		 catch (...) {}
	}
 return false;
}
//------------------------------------------------------------
bool usb_t::USB_SET_FEATURE()
{   //  запись из буфера в устройство
 try { hid_send_feature_report(handle, USB_BUFO, USB_T_RX_TX_BUF_SIZE); }
 catch (...) { return false; }
 return true;
}
//------------------------------------------------------------
bool usb_t::USB_GET_PORT(uint8_t *PS)
{   //  чтение состояния порта, 2ms
 USB_BUF_CLEAR();
 USB_BUFO[1]=0x7E;
 for (uint8_t TryCnt = 3; TryCnt > 0; TryCnt--)
	{
	 if (  USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x7E) && (USB_BUFI[2]==USB_BUFI[3]) )
		{
		 *PS = USB_BUFI[2];
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool usb_t::USB_SET_PORT(uint8_t PS)
{   //  запись состояния порта, 2ms
 USB_BUF_CLEAR();
 USB_BUFO[1]=0xE7;
 USB_BUFO[2]=PS;
 for (uint8_t TryCnt = 3; TryCnt > 0; TryCnt--)
	{
	 if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0xE7) && (USB_BUFI[2]==PS) && (USB_BUFI[3]==PS) )
		{
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool usb_t::USB_GET_FAMILY(uint8_t *FAMILY)
{   //  чтение группового кода устройства, 2ms
 USB_BUF_CLEAR();
 USB_BUFO[1]=0x1D;
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x1D) )
		{
		 *FAMILY = USB_BUFI[2];
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool usb_t::USB_GET_SOFTV(uint16_t *SV)
{   //  чтение номера версии прошивки, 2ms
 USB_BUF_CLEAR();
 USB_BUFO[1]=0x1D;
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x1D) )
		{
		 *SV = USB_BUFI[3] | ((uint16_t)USB_BUFI[4]<<8);
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool usb_t::USB_GET_ID(uint32_t *ID)
{   //  чтение ID устройства, 2ms
 USB_BUF_CLEAR();
 bool RESULT=false;
 USB_BUFO[1]=0x1D;
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x1D) )
		{
		 *ID = ((uint32_t)USB_BUFI[5]<<24) | ((uint32_t)USB_BUFI[6]<<16) | ((uint32_t)USB_BUFI[7]<<8) | USB_BUFI[8];
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool usb_t::USB_EE_RD(uint8_t ADR, uint8_t *DATA)
{   //  чтение EEPROM
 USB_BUF_CLEAR();
 USB_BUFO[1] = 0xE0;
 USB_BUFO[2] = ADR;
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0xE0) && (USB_BUFI[2]==ADR) )
		{
		 *DATA = USB_BUFI[3];
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
bool usb_t::USB_EE_WR(uint8_t ADR, uint8_t DATA)
{   //  запись EEPROM, 17ms
 USB_BUF_CLEAR();
 USB_BUFO[1] = 0x0E;
 USB_BUFO[2] = ADR;
 USB_BUFO[3] = DATA;
 for (uint8_t TryCount = 3; TryCount > 0; TryCount--)
	{
	 if ( USB_SET_FEATURE() && USB_GET_FEATURE() && (USB_BUFI[1]==0x0E) && (USB_BUFI[2]==ADR) && (USB_BUFI[3]==DATA) )
		{
		 return true;
		}
	}
 return false;
}
//------------------------------------------------------------
uint32_t usb_t::Get_Device_ID (void)
{
 return Device_ID;
}
//------------------------------------------------------------
void usb_t::Read_Device_Production_String (std::wstring &wString)
{
 for (int i=0; i<wcslen(Product_String); i++) { wString += Product_String[i]; }
}
